# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2012-10-12.      #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generator git on tinkerforge.com                   #
#############################################################

module Tinkerforge
  # Device for controlling a LCD with 4 lines a 20 characters
  class BrickletLCD20x4 < Device

    # This callback is triggered when a button is pressed. The parameter is
    # the number of the button (0 to 2).
    CALLBACK_BUTTON_PRESSED = 9

    # This callback is triggered when a button is released. The parameter is
    # the number of the button (0 to 2).
    CALLBACK_BUTTON_RELEASED = 10

    FUNCTION_WRITE_LINE = 1 # :nodoc:
    FUNCTION_CLEAR_DISPLAY = 2 # :nodoc:
    FUNCTION_BACKLIGHT_ON = 3 # :nodoc:
    FUNCTION_BACKLIGHT_OFF = 4 # :nodoc:
    FUNCTION_IS_BACKLIGHT_ON = 5 # :nodoc:
    FUNCTION_SET_CONFIG = 6 # :nodoc:
    FUNCTION_GET_CONFIG = 7 # :nodoc:
    FUNCTION_IS_BUTTON_PRESSED = 8 # :nodoc:

    # Creates an object with the unique device ID <tt>uid</tt>. This object can
    # then be added to the IP connection.
    def initialize(uid)
      super uid

      @expected_name = 'LCD 20x4 Bricklet'

      @binding_version = [1, 0, 0]

      @callback_formats[CALLBACK_BUTTON_PRESSED] = 'C'
      @callback_formats[CALLBACK_BUTTON_RELEASED] = 'C'
    end

    # Writes text to a specific line (0 to 3) with a specific position 
    # (0 to 19). The text can have a maximum of 20 characters.
    # 
    # For example: (0, 7, "Hello") will write *Hello* in the middle of the
    # first line of the display.
    # 
    # The display uses a special charset that includes all ASCII characters except
    # backslash and tilde. The LCD charset also includes several other non-ASCII characters, see
    # the `charset specification <https://github.com/Tinkerforge/lcd-20x4-bricklet/raw/master/datasheets/standard_charset.pdf>`__
    # for details. The Unicode example above shows how to specify non-ASCII characters
    # and how to translate from Unicode to the LCD charset.
    def write_line(line, position, text)
      send_request(FUNCTION_WRITE_LINE, [line, position, text], 'C C Z20', 0, '')
    end

    # Deletes all characters from the display.
    def clear_display
      send_request(FUNCTION_CLEAR_DISPLAY, [], '', 0, '')
    end

    # Turns the backlight on.
    def backlight_on
      send_request(FUNCTION_BACKLIGHT_ON, [], '', 0, '')
    end

    # Turns the backlight off.
    def backlight_off
      send_request(FUNCTION_BACKLIGHT_OFF, [], '', 0, '')
    end

    # Returns *true* if the backlight is on and *false* otherwise.
    def is_backlight_on
      send_request(FUNCTION_IS_BACKLIGHT_ON, [], '', 1, '?')
    end

    # Configures if the cursor (shown as "_") should be visible and if it
    # should be blinking (shown as a blinking block). The cursor position
    # is one character behind the the last text written with 
    # BrickletLCD20x4#write_line.
    # 
    # The default is (false, false).
    def set_config(cursor, blinking)
      send_request(FUNCTION_SET_CONFIG, [cursor, blinking], '? ?', 0, '')
    end

    # Returns the configuration as set by BrickletLCD20x4#set_config.
    def get_config
      send_request(FUNCTION_GET_CONFIG, [], '', 2, '? ?')
    end

    # Returns *true* if the button (0 to 2) is pressed. If you want to react
    # on button presses and releases it is recommended to use the
    # CALLBACK_BUTTON_PRESSED and CALLBACK_BUTTON_RELEASED callbacks.
    def is_button_pressed(button)
      send_request(FUNCTION_IS_BUTTON_PRESSED, [button], 'C', 1, '?')
    end

    # Registers a callback with ID <tt>cb</tt> to the block <tt>block</tt>.
    def register_callback(cb, &block)
      callback = block
      @registered_callbacks[cb] = callback
    end
  end
end
